package de.fhhannover.inform.trust.ifmapj_examples;

import java.util.concurrent.Semaphore;

import org.w3c.dom.Document;

import de.fhhannover.inform.trust.ifmapj.IfmapJ;
import de.fhhannover.inform.trust.ifmapj.IfmapJHelper;
import de.fhhannover.inform.trust.ifmapj.binding.IfmapStrings;
import de.fhhannover.inform.trust.ifmapj.channel.ARC;
import de.fhhannover.inform.trust.ifmapj.channel.SSRC;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapErrorResult;
import de.fhhannover.inform.trust.ifmapj.exception.IfmapException;
import de.fhhannover.inform.trust.ifmapj.exception.InitializationException;
import de.fhhannover.inform.trust.ifmapj.identifier.AccessRequest;
import de.fhhannover.inform.trust.ifmapj.identifier.IdentifierFactory;
import de.fhhannover.inform.trust.ifmapj.identifier.Identity;
import de.fhhannover.inform.trust.ifmapj.identifier.IdentityType;
import de.fhhannover.inform.trust.ifmapj.identifier.IpAddress;
import de.fhhannover.inform.trust.ifmapj.messages.PollResult;
import de.fhhannover.inform.trust.ifmapj.messages.PublishNotify;
import de.fhhannover.inform.trust.ifmapj.messages.RequestFactory;
import de.fhhannover.inform.trust.ifmapj.messages.SubscribeRequest;
import de.fhhannover.inform.trust.ifmapj.messages.SubscribeUpdate;
import de.fhhannover.inform.trust.ifmapj.metadata.EventType;
import de.fhhannover.inform.trust.ifmapj.metadata.Significance;
import de.fhhannover.inform.trust.ifmapj.metadata.StandardIfmapMetadataFactory;


/**
 * 
 */
public class PollForEvents {
	
	private IdentifierFactory ifac = IfmapJ.createIdentifierFactory();
	private RequestFactory rf = IfmapJ.createRequestFactory();
	private StandardIfmapMetadataFactory mf = IfmapJ.createStandardMetadataFactory();
	private SSRC ssrc;
	private ARC arc;
	private Semaphore mainSem, pollSem;
	
	private IpAddress ip;
	private AccessRequest ar;
	private Identity id;
	
	Document arIp, authenticatedAs, event; 
	
	public PollForEvents() {
		// prepare identifiers
		ip = ifac.createIp4("1.0.80.80");
		ar = ifac.createAr("test-18067508891:1");
		id = ifac.createIdentity(IdentityType.userName, "User1");
		
		// prepare metadata
		authenticatedAs = mf.createAuthAs();
		arIp = mf.createArIp();
		event = mf.createEvent("name", "2011-08-19T09:09:21Z", "discId", new Integer(59), new Integer(50), Significance.important, EventType.policyViolation, null, "info", "http://www.example.org");
	}
	
	private class Polling implements Runnable {

		@Override
		public void run() {
			try {
				PollResult pollResult;
			
				// main has to allow us to poll
				pollSem.acquire();
			
				// the searchResult
				System.out.print("[poll-thread] Polling for searchResult ... ");
				pollResult = arc.poll();
				if (pollResult.getSearchResults().size() < 1)
					System.out.println("Unexpected PollResult");
				else
					System.out.println("OK");
				IfmapJExamples.sleepSomeTime();
	
				// let the main thread continue
				mainSem.release();
				
				// wait for the main thread to allow us to continue
				pollSem.acquire();
				
				System.out.print("[poll-thread] Polling for updateResult ... ");
				pollResult = arc.poll();
				
				if (pollResult.getUpdateResults().size() < 1)
					System.out.println("Unexpected PollResult");
				else
					System.out.println(" OK");
				IfmapJExamples.sleepSomeTime();
			
				// let the main-thread publish notify an event metadata
				mainSem.release();
			
				// wait until the event was published
				pollSem.acquire();
				
				System.out.print("[poll-thread] Polling for notifyResult ... ");
				pollResult = arc.poll();
				
				if (pollResult.getNotifyResults().size() < 1)
					System.out.println("Unexpected PollResult");
				else
					System.out.println(" OK");
				IfmapJExamples.sleepSomeTime();
			
				// let the main thread end the session
				mainSem.release();
				
				IfmapJExamples.doLastPoll(arc);
				
			} catch (Exception e) {
				System.err.println("[poll-thread] Exception: " + e.getMessage());
				e.printStackTrace();
			}	
		}
	}

	public void run (String args[]) throws InitializationException {
		
		System.out.println("====== RUNNING POLL FOR EVENTS EXAMPLE ======");
		
		mainSem = new Semaphore(0);
		pollSem = new Semaphore(0);

		// Use basic authentication
		ssrc = IfmapJ.createSSRC(
				Config.BASIC_AUTH_SERVER_URL,
				Config.BASIC_AUTH_USER,
				Config.BASIC_AUTH_PASSWORD,
				IfmapJHelper.getTrustManagers(
				getClass().getResourceAsStream(Config.TRUST_STORE_PATH),
				Config.TRUST_STORE_PASSWORD));
		
		arc = ssrc.getArc();
		
		try {
	
			System.out.print("[main-thread] newSession ... ");
			ssrc.newSession();
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
		
			// Create a subscription beginning from 192.168.0.1
			System.out.print("[main-thread] subscribe ... ");
			SubscribeUpdate subscribeUpd = rf.createSubscribeUpdate(
					"subscription", // name
					"meta:access-request-ip",			// match-links that matches the published metadata
//					"meta:authenticated-as",			// match-links that does not match the published metadata
					3,				// maxDepth
					null,			// terminal-identifiers,
					null,			// max-size
					"meta:event or meta:access-request-ip",			// result-filter
					ar);			// start identifier
			subscribeUpd.addNamespaceDeclaration(IfmapStrings.STD_METADATA_PREFIX,
					IfmapStrings.STD_METADATA_NS_URI);
			SubscribeRequest subscribeReq = rf.createSubscribeReq(subscribeUpd);
			ssrc.subscribe(subscribeReq);
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
			
			// start the polling thread
			new Thread(new Polling()).start();
		
			// let it go
			pollSem.release();
	
			// continue if the poll thread has done its first polling
			mainSem.acquire();
			
			System.out.print("[main-thread] publishing authenticated-as... ");
			ssrc.publish(rf.createPublishReq(rf.createPublishUpdate(
					ar, id, authenticatedAs)));
			System.out.println("OK");
			
			IfmapJExamples.sleepSomeTime();
			System.out.print("[main-thread] publishing access-request-ip... ");
			ssrc.publish(rf.createPublishReq(rf.createPublishUpdate(
					ar, ip, arIp)));
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
		
			
			// let the poll-thread do it's polling
			pollSem.release();
		
			// wait until the poll-thread is done
			mainSem.acquire();

			// publish notify event metadata
			PublishNotify notify = rf.createPublishNotify(ip, event);
			
			System.out.print("[main-thread] publishing event... ");
			ssrc.publish(rf.createPublishReq(notify));
			System.out.println("OK");
			IfmapJExamples.sleepSomeTime();
		
			// let the poll-thread get the event
			pollSem.release();
		
			// wait until it's done
			mainSem.acquire();
			
			IfmapJExamples.doLastEndSession("[main-thread]", ssrc);
			IfmapJExamples.sleepSomeTime();
		} catch (IfmapException e ) {
			System.err.println(e.getMessage());
		} catch (IfmapErrorResult e) {
			System.err.println(e);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}
